/*
 * main.cpp
 *
 *  Created on: 30.03.2011
 *      Author: kostochkin
 */

#include <stdio.h>
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Light_Button.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Output.H>
#include <FL/filename.H>
#include <FL/x.H>
#include <windows.h>
#include <string.h>
#include <FL/Enumerations.H>
#include <FL/fl_ask.H>
#include "ftdi/ftdi.h"
#include <FTD2XX.H>
//#define __USE_MINGW_ANSI_STDIO 1

#define measurename "Measure"
#define variablename "cntrcounts"
#define Counter_Basic_Ref "35"
#define Counter_Basic_Time "100"

Fl_Window *window = 0;
Fl_Light_Button *lbutton = 0;
Fl_Int_Input *ref_input = 0;
Fl_Int_Input *count_times = 0;
Fl_Output *get_counts = 0;


void AnsiToUnicode (char **ansistr, wchar_t **unicodestr) {
	int len = strlen(*ansistr);
	//*unicodestr = (BSTR)malloc(lenansi);
	*unicodestr = SysAllocStringLen(NULL,len);
	MultiByteToWideChar(CP_ACP,0,*ansistr,len,*unicodestr,len);
}

HWND measurewindowhandle;
HANDLE ThreadHandle;
DWORD ThreadID;
FT_HANDLE devicehandle = 0;
char Counter_Ref[] = "-1";
bool start = false;
bool ThreadTerminate = false;
int current_times=-1;
// -Wl,-subsystem,windows
void Set_Counter_Ref() {
	char Current_Ref[1];
	strcpy(Current_Ref, ref_input->value());
	if (Current_Ref=='\0') {
		strcpy(Current_Ref, "0");
		ref_input->value("0");
	}
	if (strcmp(Counter_Ref,Current_Ref)!=0) {
		strcpy(Counter_Ref, Current_Ref);
		int ref_value = atoi(Counter_Ref) * 65535 / 2500;
		if (ref_value > 65535) {
			ref_value = 65535;
			ref_input->value("2500");
		}
		if (ref_value < 0) {
			ref_value = 0;
			ref_input->value("0");
		}
		Set_Ref(ref_value, devicehandle);
	}
}

unsigned long CountCycle (int times) {
	StartCount(times, devicehandle);
	return GetCounts(devicehandle);
}


unsigned long long StartCount64(int times) {
	unsigned long long counts = 0;
	if (times>10) do {
						//printf("main:");
						counts += (unsigned long long)CountCycle(9);
						times -= 10;
					} while (times>=10);
	if (times > 0) {
		//("%u, Ostatok:", times);
		counts += (unsigned long long)CountCycle(times-1);
	};
	//printf("%u, ",counts);
	return counts;
}

int GetTime() {
	char *time;
	time = (char*)malloc(15);
	sprintf(time,count_times->value());
	if (time=='\0') sprintf(time,"10");
	int tempint = atoi(time)/10;
	if (tempint==0) tempint=1;
	sprintf(time,"%u",tempint*10);
	count_times->value(time);
	free(time);
	return tempint;
}

void Get_Counts() {
	int times = GetTime();
	if (times<1) times=1;
	unsigned long long counts = StartCount64(times);
	if (counts!= count_error ) {
		char counts_char[20];
		sprintf(counts_char,"%I64u",counts);
		//printf("%s\n",counts_char);
		get_counts->value(counts_char);
	}
}

BOOL CALLBACK testMeasure(HWND handle, LPARAM lparam) {
	char findedwnd[20];
	GetWindowText(handle, findedwnd, 20);
	if (strstr(findedwnd, measurename) == 0) {
		return true;
	} else {
		measurewindowhandle = handle;
		return false;
	}
}


bool SendToMeasure(void) {
	char *str;
	wchar_t *unistr;
	str = (char*)malloc(40);
	sprintf(str,"%s=%s",variablename,get_counts->value());
	AnsiToUnicode(&str,&unistr);
	COPYDATASTRUCT cd;
	cd.cbData=(strlen(str)*(sizeof(wchar_t))+1);
	cd.lpData = unistr;
//	printf("Sending to measure: %s, length=%i bytes\n",str,cd.cbData);
	SendMessage( measurewindowhandle, WM_COPYDATA, 0, (LPARAM)&cd);
	free(str);
	SysFreeString(unistr);
	return true;
}

void SetButtonColor(unsigned color) {
	if (lbutton->color2() != color) {
		lbutton->color2(color);
		lbutton->redraw();
	}
}

DWORD WINAPI threadFunc(LPVOID lpvoid) {
	char devicename[] = "Counter";
	int counts = 0;
	while (1) {
		if (devicehandle!=0) {
			//printf("Handle = %u",(unsigned int)devicehandle);
			Set_Counter_Ref();
			Get_Counts();
			if (EnumWindows(testMeasure, 0)) {
				SetButtonColor(FL_RED);
			} else {
				SendToMeasure();
				SetButtonColor(FL_GREEN);
			};
		} else {
			SetButtonColor(FL_BLACK);
			FT_openbydescription2(devicename, &devicehandle);
		}
		if (ThreadTerminate) {
			FT_Close(devicehandle);
			ThreadTerminate = false;
			devicehandle=0;
			ExitThread(0);
		}
		Sleep(10);
	}
	return 0;
}

void lbutton_cb(Fl_Widget* obj, void* void1) {
	start = !start;
	if (lbutton->value() == 1) {
		ThreadHandle = CreateThread(NULL, 0, threadFunc, 0, 0, &ThreadID);
	} else {
		SetButtonColor(FL_BLUE);
		ThreadTerminate = true;
		//TerminateThread(ThreadHandle, 0);
	}

}

int main(int argc, char **argv) {
	window = new Fl_Window(160, 82);
	lbutton = new Fl_Light_Button(110, 62, 49, 20, "Run");
	lbutton->color2(FL_BLUE);
	lbutton->callback((Fl_Callback*) lbutton_cb);
	ref_input = new Fl_Int_Input(100, 1, 50, 20, "Ref level, mV ");
	ref_input->value(Counter_Basic_Ref);
	count_times = new Fl_Int_Input(100, 21, 50, 20, "Time, ms");
	count_times->value(Counter_Basic_Time);
	get_counts = new Fl_Output(60, 41, 90, 20, "Counts ");
	window->end();
	window->show(argc, argv);
	window->h();
	return Fl::run();
}
